/* **************************************************************************
//
// Nautilus Hyosung Inc. 2002 All Rights Reserved
//
// ϸ : Protocol.h
//    : Ʈũ Ʈ 
//    : 2002/09/26 űۼ(ȣ)
//
// *************************************************************************/
#ifndef _PROTOCAL_H_
#define _PROTOCAL_H_

//  
typedef unsigned short	uint16_t;
typedef unsigned int	uint32_t;
typedef unsigned int	uint_t;
typedef unsigned short	ushort_t;
typedef unsigned char	uchar_t;

// ̴ 巹
typedef struct _ETHER_ADDR {
    uchar_t ether_addr_octet[6];
} ETHER_ADDR;

// ̴ 
typedef struct _ETHER_HEADER {
    ETHER_ADDR		ether_dhost;
    ETHER_ADDR		ether_shost;
    ushort_t		ether_type;
} ETHER_HEADER;

//  
#define ETHERTYPE_PUP       0x0200      /* PUP protocol */
#define ETHERTYPE_IP        0x0800      /* IP protocol */
#define ETHERTYPE_ARP       0x0806      /* Addr. resolution protocol */
#define ETHERTYPE_REVARP    0x8035      /* Reverse ARP */
#define ETHERTYPE_AT        0x809b      /* AppleTalk protocol */
#define ETHERTYPE_AARP      0x80f3      /* AppleTalk ARP */
#define ETHERTYPE_IPV6      0x86dd      /* IPv6 */
#define ETHERTYPE_PPPOED    0x8863      /* PPPoE Discovery Stage */
#define ETHERTYPE_PPPOES    0x8864      /* PPPoE Session Stage */

// ARP  
typedef struct _ARP_HEADER {
	u_short	ar_hrd;		/* format of hardware address */
#define ARPHRD_ETHER 	1	/* ethernet hardware format */
#define ARPHRD_IEEE802	6	/* token-ring hardware format */
#define ARPHRD_ARCNET	7	/* arcnet hardware format */
#define ARPHRD_FRELAY 	15	/* frame relay hardware format */
#define ARPHRD_STRIP 	23	/* Ricochet Starmode Radio hardware format */
#define ARPHRD_IEEE1394	24	/* IEEE 1394 (FireWire) hardware format */
	u_short	ar_pro;		/* format of protocol address */
	u_char	ar_hln;		/* length of hardware address */
	u_char	ar_pln;		/* length of protocol address */
	u_short	ar_op;		/* one of: */
#define	ARPOP_REQUEST	1	/* request to resolve address */
#define	ARPOP_REPLY	2	/* response to previous request */
#define	ARPOP_REVREQUEST 3	/* request protocol address given hardware */
#define	ARPOP_REVREPLY	4	/* response giving protocol address */
#define ARPOP_INVREQUEST 8 	/* request to identify peer */
#define ARPOP_INVREPLY	9	/* response identifying peer */
	u_char	ar_sha[6];		/* sender hardware address */
	u_char	ar_spa[4];		/* sender protocol address */
	u_char	ar_tha[6];		/* target hardware address */
	u_char	ar_tpa[4];		/* target protocol address */
} ARP_HEADER;
	
// IP  
typedef struct _IP_HEADER {
    uchar_t			ip_hl:4,        /* header length */
					ip_v:4;         /* version */
    uchar_t			ip_tos;         /* type of service */
    short			ip_len;         /* total length */
    ushort_t		ip_id;         /* identification */
    short			ip_off;         /* fragment offset field */
#define IP_DF 0x4000            /* dont fragment flag */
#define IP_MF 0x2000            /* more fragments flag */
#define	IP_OFFMASK 0x1fff		/* mask for fragmenting bits */
    uchar_t			ip_ttl;         /* time to live */
    uchar_t			ip_p;           /* protocol */
    ushort_t		ip_sum;        /* checksum */
    struct in_addr	ip_src, ip_dst; /* source and dest address */
} IP_HEADER;

#define IP_MAXPACKET    4096       /* maximum packet size */

typedef uint32_t    tcp_seq;

typedef struct _ICMP_HEADER
{
  u_int8_t type;                /* message type */
  u_int8_t code;				/* type sub-code */
#define ICMP_ECHOREPLY          0       /* Echo Reply                   */
#define ICMP_DEST_UNREACH       3       /* Destination Unreachable      */
#define ICMP_SOURCE_QUENCH      4       /* Source Quench                */
#define ICMP_REDIRECT           5       /* Redirect (change route)      */
#define ICMP_ECHO               8       /* Echo Request                 */
#define ICMP_TIME_EXCEEDED      11      /* Time Exceeded                */
#define ICMP_PARAMETERPROB      12      /* Parameter Problem            */
#define ICMP_TIMESTAMP          13      /* Timestamp Request            */
#define ICMP_TIMESTAMPREPLY     14      /* Timestamp Reply              */
#define ICMP_INFO_REQUEST       15      /* Information Request          */
#define ICMP_INFO_REPLY         16      /* Information Reply            */
#define ICMP_ADDRESS            17      /* Address Mask Request         */
#define ICMP_ADDRESSREPLY       18      /* Address Mask Reply           */
#define NR_ICMP_TYPES           18
  u_int16_t checksum;
  union
  {
    struct
    {
      u_int16_t id;
      u_int16_t sequence;
    } echo;                     /* echo datagram */
    u_int32_t   gateway;        /* gateway address */
    struct
    {
      u_int16_t __unused;
      u_int16_t mtu;
    } frag;                     /* path mtu discovery */
  } un;
} ICMP_HEADER;


// TCP  
typedef struct _TCP_HEADER {
    uint16_t    th_sport;   /* source port */
    uint16_t    th_dport;   /* destination port */
    tcp_seq     th_seq;     /* sequence number */
    tcp_seq     th_ack;     /* acknowledgement number */
    uchar_t		th_x2:4,        /* (unused) */
				th_off:4;       /* data offset */
    uchar_t		th_flags;
#define TH_FIN  0x01
#define TH_SYN  0x02
#define TH_RST  0x04
#define TH_PUSH 0x08
#define TH_ACK  0x10
#define TH_URG  0x20
#define TH_ECE  0x40
#define TH_CWR  0x80
#define TH_PSHACK 0x18
    uint16_t    th_win;     /* window */
    uint16_t    th_sum;     /* checksum */
    uint16_t    th_urp;     /* urgent pointer */
} TCP_HEADER;

#define TCPHEADERSIZE 6*4

// UDP  
typedef struct _UDP_HEADER {
	uint16_t	uh_sport;		/* source port */
	uint16_t	uh_dport;		/* destination port */
	uint16_t	uh_ulen;		/* udp length */
	uint16_t	uh_sum;			/* udp checksum */
} UDP_HEADER;

typedef struct _PACKET_DATA {
	CString		index;
	char	packetdata[4096];
	
	_PACKET_DATA(CString strindex = "") 
	{
		index = strindex;
		memset(packetdata, 0x00, sizeof(packetdata));
		
	};
	
	BOOL operator == (const _PACKET_DATA& sPacket) const
	{
		if (index == sPacket.index) {
			return TRUE;
		}
		else {
			return FALSE;
		}
	};

} PACKET_DATA;

#endif
